@@ -1,29 +1,36 @@
 #' @title Geographically Weighted Elliptical Regression
 #' @import spgwr
-#' @import stats
 #' @import graphics
 #' @import methods
 #' @import sp
 #' @import spData
 #' @import maptools
+#' @importFrom GWmodel gw.dist gw.weight
 #' @name gwer
 #' @aliases print.gwer
 #' @description The function fit geographically weighted elliptical regression  model to explore the non-stationarity for a certain bandwidth and  weighting function.
 #' @param formula regression model formula as in \code{glm}.
-#' @param coords matrix of coordinates of points representing the spatial positions of the observations.
-#' @param bandwidth value of the selected bandwidth used in the weighting function (see \code{gwer.sel} for bandwidth optimization).
-#' @param gweight geographical weighting function, at present \code{gwr.Gauss()} is default.
+#' @param bandwidth value of the selected bandwidth used in the weighting function (see \code{bw.gwer} for bandwidth optimization).
+#' @param kernel function chosen as follows:
+#' gaussian: wgt = exp(-.5*(vdist/bw)^2);
+#' exponential: wgt = exp(-vdist/bw);
+#' bisquare: wgt = (1-(vdist/bw)^2)^2 if vdist < bw, wgt=0 otherwise;
+#' tricube: wgt = (1-(vdist/bw)^3)^3 if vdist < bw, wgt=0 otherwise;
+#' boxcar: wgt=1 if dist < bw, wgt=0 otherwise
+#' @param p the power of the Minkowski distance, default is 2, i.e. the Euclidean distance
+#' @param theta an angle in radians to rotate the coordinate system, default is 0
+#' @param dMat a pre-specified distance matrix, it can be calculated by the function gw.dist
+#' @param regression.points a Spatial*DataFrame object, i.e. SpatialPointsDataFrame or SpatialPolygonsDataFrame as defined in package sp; Note that no diagnostic information will returned if it is assigned.
 #' @param adapt defines the type of bandwidth used. either NULL (default) or a proportion between 0 and 1 of observations to include in weighting scheme (k-nearest neighbours).
-#' @param hatmatrix if TRUE, return the hatmatrix as a component of the result, ignored if fit.points given
+#' @param hatmatrix if TRUE, return the hatmatrix as a component of the result.
 #' @param spdisp if TRUE dispersion parameter varies geographically.
-#' @param family a description of the error distribution to be used in the model (see \code{elliptical.family} for details of family functions).
+#' @param family a description of the error distribution to be used in the model (see \code{\link{family.elliptical}} for details of family functions).
 #' @param data model data frame, or may be a SpatialPointsDataFrame or SpatialPolygonsDataFrame as defined in package \pkg{sp}.
 #' @param dispersion an optional fixed value for dispersion parameter.
 #' @param weights an optional numeric vector of weights to be used in the fitting process.
 #' @param subset an optional numeric vector specifying a subset of observations to be used in the fitting process.
-#' @param fit.points an object containing the coordinates of fit points, often an object from package \pkg{sp}. If missing, the coordinates given through the data argument object, or the coords argument are used.
 #' @param na.action a function which indicates what should happen when the data contain NAs (see \code{glm}).
-#' @param method the method to be used in fitting local models. The default method "gwer.fit" uses Fisher's scoring method. The alternative "model.frame" returns the model frame and does no fitting.
+#' @param method the method to be used in fitting local models. The default method "bw.gwer" uses Fisher's scoring method. The alternative "model.frame" returns the model frame and does no fitting.
 #' @param longlat TRUE if point coordinates are longitude-latitude decimal degrees, in which case distances are measured in kilometers. If x is a SpatialPoints object, the value is taken from the object itself.
 #' @param control a list of parameters for controlling the fitting process. For \code{elliptical} this is passed by \code{glm.control}.
 #' @param model a logical value indicating whether model frame should be included as a component of the return.
@@ -32,10 +39,9 @@
 #' @param contrasts an optional list. See the \code{contrasts.arg} of \code{model.matrix.default}.
 #' @param parplot if TRUE the parameters boxplots are plotted.
 #' @param offset this can be used to specify an a priori known component to be included in the linear predictor during fitting as in \code{glm}.
-#' @param type a character string that indicates the type of residuals should consider as return.
 #' @param ... arguments to be used to form the default control argument if it is not supplied directly.
 #' @return returns an object of class \dQuote{gwer}, a list with follow components:   
-#' \item{SDF}{a SpatialPointsDataFrame (may be gridded) or SpatialPolygonsDataFrame object (see package \pkg{sp}) with fit.points, weights, GWR coefficient estimates, dispersion and the residuals of \code{type} in its \code{data} slot.}
+#' \item{SDF}{a SpatialPointsDataFrame (may be gridded) or SpatialPolygonsDataFrame object (see package \pkg{sp}) with fit.points, weights, GWR coefficient estimates, dispersion and the residuals in its \code{data} slot.}
 #' \item{coef}{the matrices of coefficients, standard errors and significance values for parameters hypothesis test.}
 #' \item{dispersion}{either the supplied argument or the estimated dispersion with standard error.}
 #' \item{hat}{hat matrix of the geographically weighted elliptical model.}
@@ -48,96 +54,87 @@
 #' \item{family}{the \code{family} object used.}
 #' \item{flm}{a matrix with the fitted values for all local elliptical models.}
 #' \item{adapt}{the \code{adapt} object used.}
-#' \item{gweight}{the \code{gweights} object used.}
+#' \item{kernel}{the \code{kernel} object used.}
 #' \item{spdisp}{the \code{spdisp} object used.}
 #' \item{this.call}{the function call used.}
-#' \item{fp.given}{the \code{fp.given} object used.}
 #' \item{longlat}{the \code{longlat} object used.}
-#' \item{type}{the \code{type} residuals for the object used.}
 #' @references Brunsdon, C., Fotheringham, A. S. and Charlton, M. E. (1996). 
 #' Geographically weighted regression: a method for exploring spatial nonstationarity.
-#' Geographical analysis, 28(4), 281-298. \url{https://doi.org/10.1111/j.1538-4632.1996.tb00936.x}
+#' Geographical analysis, 28(4), 281-298. \doi{10.1111/j.1538-4632.1996.tb00936.x}
 #' @references Cysneiros, F. J. A., Paula, G. A., and Galea, M. (2007). Heteroscedastic 
 #' symmetrical linear models. Statistics & probability letters, 77(11), 1084-1090. 
-#' \url{https://doi.org/10.1016/j.spl.2007.01.012} 
+#' \doi{10.1016/j.spl.2007.01.012} 
 #' @references Fang, K. T., Kotz, S. and NG, K. W. (1990, ISBN:9781315897943).
 #' Symmetric Multivariate and Related Distributions. London: Chapman and Hall.
-#' @seealso \code{\link{gwer.sel}}, \code{\link{elliptical}}, \code{\link{family.elliptical}}
+#' @seealso \code{\link{bw.gwer}}, \code{\link{elliptical}}, \code{\link{family.elliptical}}
 #' @keywords Geographically weighted regression
 #' @keywords Bandwidth optimization
 #' @keywords Elliptical model
 #' @examples
-#' data(columbus, package="spData")
-#' fit.lm <- lm(CRIME ~ INC, data=columbus)
-#' summary(fit.lm)
-#' gwer.bw <- gwer.sel(CRIME ~ INC, data=columbus, family = Normal(),
-#'                  coords=cbind(columbus$X, columbus$Y), method = 'aic')
-#' gwer.fit <- gwer(CRIME ~ INC, family = Normal(), bandwidth = gwer.bw, hatmatrix = TRUE, 
-#'                  spdisp = TRUE, parplot = TRUE, data=columbus, method = "gwer.fit",
-#'                  coords=cbind(columbus$X, columbus$Y))
-#' print(gwer.fit) 
 #' \donttest{
-#' data(columbus, package="spData")
-#' fit.elliptical <- elliptical(CRIME ~ INC, family = Student(df=4), data=columbus)
-#' summary(fit.elliptical)
-#' gwer.bw <- gwer.sel(CRIME ~ INC, data=columbus, family = Student(df=4),
-#'                  coords=cbind(columbus$X, columbus$Y), method = 'aic')
-#' gwer.fitt <- gwer(CRIME ~ INC, family = Student(df=4), bandwidth = gwer.bw, hatmatrix = TRUE,
-#'                  spdisp = TRUE, parplot = TRUE, data=columbus, method = "gwer.fit",
-#'                  coords=cbind(columbus$X, columbus$Y))
-#' print(gwer.fitt)  
+#' data(georgia, package = "spgwr")
+#' fit.formula <- PctBach ~ TotPop90 + PctRural + PctFB + PctPov
+#' gwer.bw.t <- bw.gwer(fit.formula, data = gSRDF, family = Student(3), adapt = TRUE)
+#' gwer.fit.t <- gwer(fit.formula, data = gSRDF, family = Student(3), bandwidth = gwer.bw.t, 
+#'                    adapt = TRUE, parplot = FALSE, hatmatrix = TRUE, spdisp = TRUE, 
+#'                    method = "gwer.fit")
+#' print(gwer.fit.t) 
 #' }
 #' @rdname gwer
 #' @export
 
-gwer <- function (formula, data, coords, bandwidth, gweight = gwr.Gauss, 
-                  adapt = NULL, hatmatrix = FALSE, fit.points, family = Normal, longlat = NULL, 
+gwer <- function (formula, data, regression.points, bandwidth, kernel = "bisquare", 
+                  p = 2, theta = 0, adapt = NULL, hatmatrix = FALSE, family = Normal, longlat = NULL, dMat,
                   weights, dispersion = NULL, subset, na.action = "na.fail", method = "gwer.fit",
                   control = glm.control(epsilon = 1e-04, maxit = 100, trace = F), model = FALSE,
-                  x = FALSE, y = TRUE, contrasts = NULL, offset, type='pearson', spdisp = FALSE,
-                  parplot = FALSE,...)
+                  x = FALSE, y = TRUE, contrasts = NULL, offset, spdisp = TRUE, parplot = FALSE,...)
 {
   call <- match.call()
   this.call <- match.call()
   dist <- as.character(call$family)[1]
   user.def <- F
   
-  if (!charmatch(dist, c("Normal", "Cauchy", "Student", "Gstudent", 
-                         "LogisI", "LogisII", "Glogis", "Cnormal", "Powerexp"), 
-                 nomatch = F)) 
-    dist <- match.arg(dist, c("Normal", "Cauchy", "Student", 
-                              "Gstudent", "LogisI", "LogisII", "Glogis", "Cnormal", 
-                              "Powerexp"))
+  if (!charmatch(dist, c("Normal", "Cauchy", "Student", "Gstudent", "LogisI", "LogisII", "Glogis", "Cnormal", "Powerexp"),nomatch = F)) 
+    dist <- match.arg(dist, c("Normal", "Cauchy", "Student", "Gstudent", "LogisI", "LogisII", "Glogis", "Cnormal", "Powerexp"))
   else user.def <- T
   
-  resid_name <- paste(type, "resids", sep = "_")
+  resid_name <- "residuals"
   p4s <- as.character(NA)
   Polys <- NULL
   if (is(data, "SpatialPolygonsDataFrame")) 
     Polys <- as(data, "SpatialPolygons")
   
-  if (is(data, "Spatial")) {
-    if (!missing(coords)) 
-      warning("data is Spatial* object, ignoring coords argument")
-    coords <- coordinates(data)
-    p4s <- proj4string(data)
-    if (is.null(longlat) || !is.logical(longlat)) {
-      if (!is.na(is.projected(data)) && !is.projected(data)) {
-        longlat <- TRUE
-      }
-      else {
-        longlat <- FALSE
-      }
+  if (missing(regression.points)) {
+    rp.given <- FALSE
+    regression.points <- data
+    rp.locat <- coordinates(data)
+    hatmatrix <- T
+  }
+  else {
+    rp.given <- TRUE
+    hatmatrix <- F
+    if (is(regression.points, "Spatial")) {
+      rp.locat <- coordinates(regression.points)
     }
+    else if (is.numeric(regression.points) && dim(regression.points)[2] == 
+             2) 
+      rp.locat <- regression.points
+    else {
+      warning("Output loactions are not packed in a Spatial object,and it has to be a two-column numeric vector")
+      rp.locat <- dp.locat
+    }
+  }
+  if (is(data, "Spatial")) {
+    p4s <- proj4string(data)
+    dp.locat <- coordinates(data)
     data <- as(data, "data.frame")
   }
+  else {
+    stop("Given regression data must be Spatial*DataFrame")
+  }
   
   if (is.null(longlat) || !is.logical(longlat)) 
     longlat <- FALSE
-  if (missing(coords)) 
-    stop("Observation coordinates have to be given")
-  if (is.null(colnames(coords))) 
-    colnames(coords) <- c("coord.x", "coord.y")
   if (is.character(family)) 
     family <- get(family, mode = "function", envir = parent.frame())
   if (is.function(family)) 
@@ -153,7 +150,7 @@ gwer <- function (formula, data, coords, bandwidth, gweight = gwr.Gauss,
   if (!charmatch(method, c("model.frame", "gwer.fit"), F)) 
     stop(paste("\n unimplemented method:", method))
   m <- match.call(expand.dots = FALSE)
-  m$family <- m$method <- m$control <- m$model <- m$dispersion <- m$x <- m$y <- m$bandwidth  <- m$gweight <- m$adapt <- m$longlat <-m$contrasts <- m$offset <- m$hatmatrix <- m$spdisp <- m$parplot <- m$... <- NULL
+  m$family <- m$method <- m$control <- m$model <- m$dispersion <- m$x <- m$y <- m$bandwidth  <- m$kernel <- m$adapt <- m$longlat <-m$contrasts <- m$offset <- m$hatmatrix <- m$spdisp <- m$parplot <- m$... <- NULL
   m[[1]] <- as.name("model.frame")
   m <- eval(m, sys.parent())
   dp.n <- length(model.extract(m, "response"))
@@ -179,7 +176,6 @@ gwer <- function (formula, data, coords, bandwidth, gweight = gwr.Gauss,
   if (length(offset) == 1 && offset == 0) 
     offset <- rep(0, nobs)
   
-  
   w <- model.extract(m, weights)
   wzero <- rep(F, nrow(m))
   if (!length(w)) 
@@ -208,66 +204,53 @@ gwer <- function (formula, data, coords, bandwidth, gweight = gwr.Gauss,
   elliptical.fitter <- get(method)
   offset4fit <- offset
   
-  fit <- elliptical.fitter(X = X, Y = Y,  offset = offset4fit, family = family, 
+  fit <- elliptical.fitter(X = X, Y = Y, offset = offset4fit, family = family, 
                            dispersion = dispersion, maxit = control$maxit, 
                            epsilon = control$epsilon, trace = control$trace, ...)
   if (!spdisp)
-    dispersiongwr <- fit$dispersion
+    dispersiongwr <- dispersion <- fit$dispersion
   if (hatmatrix) 
     se.fit <- TRUE
-  if (missing(fit.points)) {
-    fp.given <- FALSE
-    fit.points <- coords
-    colnames(fit.points) <- colnames(coords)
-  }
-  else fp.given <- TRUE
-  griddedObj <- FALSE
-  if (is(fit.points, "Spatial")) {
-    Polys <- NULL
-    if (is(fit.points, "SpatialPolygonsDataFrame")) {
-      Polys <- Polygons(fit.points)
-      fit.points <- coordinates(fit.points)
-    }
-    else {
-      griddedObj <- gridded(fit.points)
-      fit.points <- coordinates(fit.points)
-    }
-  }
-  n <- NROW(fit.points)
-  rownames(fit.points) <- NULL
-  if (is.null(colnames(fit.points))) 
-    colnames(fit.points) <- c("x", "y")
-
+  
+  n <- NROW(rp.locat)
+  rownames(rp.locat) <- NULL
+  if (is.null(colnames(rp.locat))) 
+    colnames(rp.locat) <- c("x", "y")
   p <- NCOL(X)
 
-  if (NROW(X) != NROW(coords)) 
+  if (NROW(X) != NROW(regression.points)) 
     stop("Input data and coordinates have different dimensions")
-  
-  if (is.null(adapt)) {  
-    if (!missing(bandwidth)) {
-      bw <- bandwidth
-      bandwidth <- rep(bandwidth, nobs)
+
+  DM.given <- F
+  if (missing(dMat)) {
+    DM.given <- F
+    DM1.given <- F
+    if (dp.n + n <= 5000) {
+      dMat <- gw.dist(dp.locat = dp.locat, rp.locat = rp.locat, 
+                      p = p, theta = theta, longlat = longlat)
+      DM.given <- T
+    }
+    else {
+      dMat <- matrix(0, 1, 1)
     }
-    else stop("Bandwidth must be given for non-adaptive weights")
   }
   else {
-    bandwidth <- gw.adapt(dp = coords, fp = fit.points, quant = adapt, 
-                          longlat = longlat)
-    bw <- bandwidth
+    DM.given <- T
+    DM1.given <- T
+    dim.dMat <- dim(dMat)
+    if (dim.dMat[1] != dp.n || dim.dMat[2] != n) 
+      stop("Dimensions of dMat are not correct")
   }
-  
-  if (any(bandwidth < 0)) 
-    stop("Invalid bandwidth")
 
   v_resids <- numeric(n) ; v_fitteds <- numeric(n)
 
   if (any(wzero)) {
     nas <- is.na(fit$coef)
     fitted[wpos] <- fit$fitted.values/w[wpos]
-    fitted[wzero] <- X.org[wzero, !nas] %*% as.vector(fit$coef[!nas]) + 
+    fitted[wzero] <- X.org[wzero, !nas] %*% as.vector(fit$coefficients[!nas]) + 
       if (length(offset.org) > 1) 
         offset.org[wzero]
-    else 0
+      else 0
     fit$fitted.values <- fitted
     resid[wpos] <- fit$resid
     resid[wzero] <- (Y.org[wzero] - fitted[wzero])/sqrt(fit$dispersion)
@@ -291,7 +274,7 @@ gwer <- function (formula, data, coords, bandwidth, gweight = gwr.Gauss,
   asgn <- attr(if (exists("X.org", frame = sys.nframe())) X.org else X, 
                "assign")
   if (rank < p) {
-    nas <- is.na(fit$coef)
+    nas <- is.na(fit$coefficients)
     pasgn <- asgn[!nas]
     if (df.residuals > 0) 
       fit$assign.residual <- (rank + 1):length(Y)
@@ -299,31 +282,38 @@ gwer <- function (formula, data, coords, bandwidth, gweight = gwr.Gauss,
     fit$x.assign <- asgn
   }
   
-  gweights <- matrix(0,n,n) ;  resid.ord <- matrix(0,n,1)  #; Vi <- matrix(0,n,n)
-  local.fitted <- matrix(0,n,n) ; sum.w <- numeric(n)  
+  gweights <- matrix(0,n,n) ; local.fitted <- matrix(0,n,n) ; sum.w <- numeric(n) ; resid.ord <- matrix(0,n,1)
+    
 
   if(spdisp){ 
     dispersiongwr <- numeric(n) 
     gwr.b <- matrix(nrow = nobs, ncol = ncol(X)+1)
     stderror <- matrix(0,n,p+1) ; zvalue <- matrix(0,n,p) ; pvalue <- matrix(0,n,p)
-    colnames(stderror) <- c(paste(colnames(X),rep(".se",p), sep = ""), "dispersion.se")
     colnames(gwr.b) <- c(colnames(X), "dispersion")
+    colnames(stderror) <- c(paste(colnames(X),rep(".se",p), sep = ""), "dispersion.se")
   } else {
     gwr.b <- matrix(nrow = nobs, ncol = ncol(X))
     stderror <- matrix(0,n,p) ; zvalue <- matrix(0,n,p) ; pvalue <- matrix(0,n,p)
-    colnames(stderror) <- paste(colnames(X),rep(".se",p), sep = "")
     colnames(gwr.b) <- c(colnames(X))    
+    colnames(stderror) <- paste(colnames(X),rep(".se",p), sep = "")
   }
   colnames(zvalue) <- colnames(pvalue) <- c(colnames(X)) 
   
-  if (!fp.given && hatmatrix)
+  if (hatmatrix)
     Hat <- matrix(0,n,n)
   
   for (i in 1:n) {
-    dxs <- spDistsN1(coords, fit.points[i, ], longlat = longlat)
-    if (any(!is.finite(dxs))) 
-      dxs[which(!is.finite(dxs))] <- .Machine$double.xmax/2
-    w.i <- gweight(dxs^2, bandwidth[i])
+    if (DM.given) 
+      dist.vi <- dMat[, i]
+    else {
+      if (rp.given) 
+        dist.vi <- gw.dist(dp.locat, rp.locat, focus = i, 
+                           p, theta, longlat)
+      else dist.vi <- gw.dist(dp.locat = dp.locat, focus = i, 
+                              p = p, theta = theta, longlat = longlat)
+    }
+
+    w.i <- gw.weight(dist.vi, bandwidth, kernel, adapt)
     if (any(w.i < 0 | is.na(w.i))) 
       stop(paste("Invalid weights for i:", i))
     lm.i <- elliptical.fitter(X = X, Y = Y, gweights = w.i, offset = offset4fit, 
@@ -331,25 +321,25 @@ gwer <- function (formula, data, coords, bandwidth, gweight = gwr.Gauss,
                               maxit = control$maxit, epsilon = control$epsilon, 
                               trace = control$trace, ...)
     sum.w[i] <- sum(w.i)
-    v_fitteds[i] <- fitted.values(lm.i)[i]
+    v_fitteds[i] <- lm.i$fitted.values[i]
     if(spdisp){ 
       dispersiongwr[i] <- lm.i$dispersion
-      gwr.b[i, ] <- c(coefficients(lm.i),dispersiongwr[i])
+      gwr.b[i, ] <- c(coefficients(lm.i), dispersiongwr[i])
     } else{
       gwr.b[i, ] <- coefficients(lm.i)
     }
     
     if (any(wzero)) {
-      nas <- is.na(fit$coef)
+      nas <- is.na(lm.i$coefficients)
       fitted.i[wpos] <- lm.i$fitted.values/w[wpos]
-      fitted.i[wzero] <- X.org[wzero, !nas] %*% as.vector(fit$coef[!nas]) + 
+      fitted.i[wzero] <- X.org[wzero, !nas] %*% as.vector(lm.i$coefficients[!nas]) + 
         if (length(offset.org) > 1) 
           offset.org[wzero]
       else 0
       lm.i$fitted.values <- fitted.i
       resid.i[wpos] <- lm.i$resid
       resid.i[wzero] <- (Y.org[wzero] - fitted.i[wzero])/sqrt(lm.i$dispersion)
-      lm.i$residuals <- resid.i ; resid.ord <- lm.i$residuals
+      lm.i$residuals <- resid.i
       q1.i[wpos] <- lm.i$q1
       q2.i[wpos] <- lm.i$q2
       q1.i[wzero] <- family$g1(resid[wzero], df = family$df, 
@@ -370,14 +360,14 @@ gwer <- function (formula, data, coords, bandwidth, gweight = gwr.Gauss,
     asgn.i <- attr(if (exists("X.org", frame = sys.nframe())) X.org else X, 
                    "assign")
     if (rank < p) {
-      nas.i <- is.na(lm.i$coef)
+      nas.i <- is.na(lm.i$coefficients)
       pasgn.i <- asgn[!nas]
       if (df.residuals > 0) 
         lm.i$assign.residual <- (rank.i + 1):length(Y)
       lm.i$R.assign <- pasgn.i
       lm.i$x.assign <- asgn.i
     }
-    lm.i <- c(lm.i, list(assign = asgn, df.residuals = df.residuals, 
+    lm.i <- c(lm.i, list(assign = asgn.i, df.residuals = df.residuals, 
               family = family, user.def = user.def, formula = as.vector(attr(Terms, 
               "formula")), terms = Terms, contrasts = attr(X, 
               "contrasts"), control = control, call = call))
@@ -396,65 +386,48 @@ gwer <- function (formula, data, coords, bandwidth, gweight = gwr.Gauss,
     attr(lm.i, "class") <- c("elliptical", "glm", "lm")
     
     local.fitted[i,] <- lm.i$fitted.values
-    if (!fp.given) 
-      v_resids[i] <- residuals.gwer(lm.i, type = type)[i]
-    else is.na(v_resids[i]) <- TRUE
 
-    
     R <- lm.i$R[(1:p), (1:p)]
     covun <- solve(qr(R))
     rowlen <- sqrt(diag(covun))
 
- 
-    if (!fp.given && hatmatrix) {   
+    if (hatmatrix) {
+      Hat[i,] <- lm.i$Hat[i,] ; nu1 <- sum(diag(Hat)) ; nu2 <- sum(diag(t(Hat)%*%Hat)) ; alpha.pv <- (nu1/p)
       if(spdisp){
         stderror[i, ] <- c(rowlen[1:p] * sqrt(lm.i$dispersion/lm.i$scale), sqrt(4*lm.i$dispersion^2/(sum(lm.i$gweights)*lm.i$scaledispersion)))
         zvalue[i, ] <- c(gwr.b[i, 1:p])/stderror[i, 1:p]
-        pvalue[i, ] <- 2 * pnorm(-abs(zvalue[i, ]))
+        pvalue[i, ] <- 2 * pnorm(-abs(zvalue[i, ]))#/(nu1/p)
       } else {
         stderror[i, ] <- rowlen[1:p] * sqrt(lm.i$dispersion/lm.i$scale)
         zvalue[i, ] <- c(gwr.b[i, ])/stderror[i, ]
-        pvalue[i, ] <- 2 * pnorm(-abs(zvalue[i, ]))
+        pvalue[i, ] <- 2 * pnorm(-abs(zvalue[i, ]))#/(nu1/p)
       }
-      Hat[i,] <- X[i,] %*% solve(t(X) %*% diag(w.i) %*% X) %*% t(X) %*% diag(w.i)    
     }
   }
   
-  
   if (any(wzero)) {
     fittedgwr[wpos] <- v_fitteds/w[wpos]
-    fittedgwr[wzero] <- X.org[wzero, !nas] %*% as.vector(fit$coef[!nas]) + 
+    fittedgwr[wzero] <- X.org[wzero, !nas] %*% as.vector(fit$coefficients[!nas]) + 
       if (length(offset.org) > 1) 
         offset.org[wzero]
     else 0
-    resid.ord[wpos] <- v_resids
-    resid.ord[wzero] <- (Y.org[wzero] - fitted[wzero])/sqrt(dispersiongwr)
+    residualgwr[wpos] <- resid.ord
+    residualgwr[wzero] <- (Y.org[wzero] - fitted[wzero])/sqrt(dispersiongwr)
   }	
   else {
     fittedgwr <- v_fitteds/w
-    resid.ord <- v_resids
+    residualgwr <- resid.ord
   }
   
   df <- data.frame(sum.w = sum.w, gwr.b)
-  if (!fp.given && hatmatrix)
+  if (hatmatrix)
     df <- data.frame(sum.w = sum.w, gwr.b, stderror)
   df[[resid_name]] <- v_resids
-  SDF <- SpatialPointsDataFrame(coords = fit.points, data = df, 
+  SDF <- SpatialPointsDataFrame(coords = rp.locat, data = df, 
                                 proj4string = CRS(p4s))
-  if (griddedObj) {
-    gridded(SDF) <- TRUE
-  }
-  else {
-    if (!is.null(Polys)) {
-      df <- data.frame(SDF@data)
-      rownames(df) <- sapply(slot(Polys, "polygons"), function(i) slot(i, "ID"))
-      SDF <- SpatialPolygonsDataFrame(Sr = Polys, data = df)
-    }
-  }
-  
 
-  if (!fp.given && hatmatrix) {
-    nu1 <- sum(diag(Hat)) ; nu2 <- sum(diag(t(Hat)%*%Hat)) ; res <- (Y - fittedgwr)/sqrt(dispersiongwr)
+  if (hatmatrix) {
+    res <- (Y - fittedgwr)/sqrt(dispersiongwr)
     logLik <- -0.5 * sum(log(dispersiongwr)) + sum(family$g0(res, df = family$df, s = family$s, r = family$r,
                                                              alpha = family$alpha, mp = family$mp, epsi = family$epsi,
                                                              sigmap = family$sigmap, k = family$k))
@@ -468,7 +441,7 @@ gwer <- function (formula, data, coords, bandwidth, gweight = gwr.Gauss,
                     logLik = - 2*logLik, AIC = AIC, AICc = AICc, BIC = BIC)
     
     local.coef <- list(est = gwr.b, se = stderror, pvalue = pvalue)
-  }else {
+  } else {
     local.coef <- list(est = gwr.b)
     results <- Hat <- NULL
   }
@@ -502,10 +475,10 @@ gwer <- function (formula, data, coords, bandwidth, gweight = gwr.Gauss,
   attr(fit, "class") <- c("elliptical", "glm", "lm")
 
   z <- list(SDF = SDF, coef = local.coef, dispersion = dispersiongwr, hat = Hat, lm = fit, 
-            results = results, bandwidth = bw, fitted = fittedgwr, hatmatrix = hatmatrix, 
-            gweights = gweights, family = fit$family, flm = local.fitted, adapt = adapt,
-            gweight = deparse(substitute(gweight)), spdisp = spdisp, this.call = this.call, 
-            fp.given = fp.given, longlat = longlat, type = type)
+            results = results, bandwidth = bandwidth, fitted = fittedgwr, residual = residualgwr, 
+            hatmatrix = hatmatrix, gweights = gweights, family = fit$family, flm = local.fitted, 
+            adapt = adapt, kernel = kernel, spdisp = spdisp, alpha.pv = alpha.pv, this.call = this.call, 
+            longlat = longlat, control = control, offset = offset)
   
   class(z) <- "gwer"
   invisible(z)
@@ -522,7 +495,7 @@ gwer.fit <- function (X, Y, gweights=NULL, offset, family, dispersion,
     gweights <- rep(1,n)
   if (is.null(offset)) 
     offset <- rep(0, n)
-
+  
   p <- ncol(X)
   aux.model <- glm.fit(x = X, y = Y, offset = offset, weights = gweights,
                        family = gaussian())
@@ -555,10 +528,9 @@ gwer.fit <- function (X, Y, gweights=NULL, offset, family, dispersion,
     options(warn = 0)
   }
   
-  
   linearconv <- TRUE
   iter <- 1
-  error2 <- error3 <- 0
+  error1 <- error2 <- error3 <- 0
   repeat {
     if (trace) 
       cat("\n iteration", iter, ":")
@@ -583,7 +555,7 @@ gwer.fit <- function (X, Y, gweights=NULL, offset, family, dispersion,
                            family = gaussian())
       attr(aux.model, "class") <- c("glm", "lm")
       new.start <- coef(aux.model)
-      }
+    }
     error1 <- max(abs((new.start - start)/start))
     start <- new.start
     
@@ -600,10 +572,10 @@ gwer.fit <- function (X, Y, gweights=NULL, offset, family, dispersion,
     args <- abs.res/sqrt(dispersion)
     if (trace) {
       loglik <- -0.5 * length(abs.res) * log((dispersion)) + 
-        sum(family$g0(abs.res/sqrt(dispersion), df = family$df, 
-                      s = family$s, r = family$r, alpha = family$alpha, 
-                      mp = family$mp, epsi = family$epsi, sigmap = family$sigmap, 
-                      k = family$k))
+                sum(family$g0(abs.res/sqrt(dispersion), df = family$df, 
+                s = family$s, r = family$r, alpha = family$alpha, 
+                mp = family$mp, epsi = family$epsi, sigmap = family$sigmap, 
+                k = family$k))
       cat(" log-likelihood =", signif(loglik, 6))
     }
     error3 <- sqrt(sum((args - old.args)^2)/max(1e-20, sum(old.args^2)))
@@ -636,14 +608,18 @@ gwer.fit <- function (X, Y, gweights=NULL, offset, family, dispersion,
     cat("\n --- negative iterative weights returned! --- \n")
   
   if (is.null.disp) {
-    rank <- dim(X)[2]
+    Xd <- cbind(X, residuals) ; rank <- dim(X)[2]
     Rnames <- dimnames(X)[[2]]
-    Xd <- cbind(X, residuals)
+    dimnames(Xd)[[2]] <- c(Rnames, "scale")
+  } else {
+    Xd <- cbind(X) ; rank <- dim(X)[2]
+    Rnames <- dimnames(X)[[2]]
+    dimnames(Xd)[[2]] <- Rnames
   }
-  dimnames(Xd)[[2]] <- c(Rnames, "scale")
+
   nn <- is.null(Rnames)
   Rnames <- list(dimnames(Xd)[[2]], dimnames(Xd)[[2]])
-  R <- t(Xd) %*% diag(gweights) %*% Xd
+  R <- t(Xd)%*%diag(as.vector(gweights))%*%Xd
   if (is.null.disp) 
     R[rank + 1, rank + 1] <- R[rank + 1, rank + 1] + length(residuals)
   attributes(R) <- list(dim = dim(R))
@@ -654,9 +630,16 @@ gwer.fit <- function (X, Y, gweights=NULL, offset, family, dispersion,
                   r = family$r, alpha = family$alpha, mp = family$mp, 
                   epsi = family$epsi, sigmap = family$sigmap, k = family$k))
   names(loglik) <- NULL
+  w.2.h <- as.vector(-2 * w.1 * gweights)
+  Hat <- X %*% solve(t(X) %*% diag(w.2.h) %*% X) %*% t(X) %*% diag(w.2.h)
+  C <- solve(t(X) %*% diag(w.2.h) %*% X) %*% t(X) %*% diag(w.2.h)
+  H <- X %*% solve(t(X) %*% diag(as.vector(gweights)) %*% X) %*% t(X) %*% diag(as.vector(gweights))
+  Haux <- solve(t(X) %*% diag(as.vector(gweights)) %*% X) %*% t(X) %*% diag(as.vector(gweights))
+  
   fit <- list(coefficients = coefs, dispersion = dispersion, gweights = gweights,
               fixed = !is.null.disp, residuals = residuals, fitted.values = fitted, 
-              loglik = loglik, convergence = linearconv, Wg = family$g1(residuals, df = family$df, 
+              loglik = loglik, convergence = linearconv, Hat = Hat, Ci = C, H = H, Haux = Haux,
+              Wg = family$g1(residuals, df = family$df, 
               r = family$r, s = family$s, alpha = family$alpha, 
               mp = family$mp, epsi = family$epsi, sigmap = family$sigmap, 
               k = family$k), Wgder = family$g5(residuals, df = family$df, 
@@ -697,15 +680,12 @@ print.gwer <- function(x, ...) {
     stop("not a gwer object")
   cat("Call:\n")
   print(x$this.call)
-  cat("Kernel function:", x$gweight, "\n")
+  cat("Kernel function:", x$kernel, "\n")
   n <- length(x$lm$residuals)
   if (is.null(x$adapt)) cat("Fixed bandwidth:", x$bandwidth, "\n")
-  else cat("Adaptive quantile: ", x$adapt, " (about ", 
-           floor(x$adapt*n), " of ", n, " data points)\n", sep="")
-  if (x$fp.given) cat("Fit points: ", nrow(x$SDF), "\n", sep="")
+  else cat("Adaptive quantile: ", x$adapt, " (about ", floor(x$adapt*n), " of ", n, " data points)\n", sep="")
   m <- length(x$lm$coefficients)
-  cat("Summary of GWER coefficient estimates at ",
-      ifelse(x$fp.given, "fit", "data"), " points:\n", sep="")
+
   if(!x$spdisp){
     df0 <- as(x$SDF, "data.frame")[,(1+(1:m)), drop=FALSE]
   } else {
@@ -717,14 +697,12 @@ print.gwer <- function(x, ...) {
   }
   CM <- t(apply(df0, 2, summary))[,c(1:3,5,6)]
   if (is.null(dim(CM))) CM <- t(as.matrix(CM))
-  if (!x$fp.given) {
-    if(!x$spdisp){
-      CM <- cbind(CM, c(coefficients(x$lm)))
-    } else {
-      CM <- cbind(CM, c(coefficients(x$lm),x$lm$dispersion))
-    }
-    colnames(CM) <- c(colnames(CM)[1:5], "Global")
+  if(!x$spdisp){
+     CM <- cbind(CM, c(coefficients(x$lm)))
+  } else {
+    CM <- cbind(CM, c(coefficients(x$lm),x$lm$dispersion))
   }
+  colnames(CM) <- c(colnames(CM)[1:5], "Global")
   printCoefmat(CM)
   cat("Number of data points:", n, "\n")
   if(!x$spdisp)
@@ -738,9 +716,6 @@ print.gwer <- function(x, ...) {
     cat("Effective degrees of freedom (model: traceS):",
         (n - x$results$nu1), "\n")
     cat("AIC:", x$results$AIC, " AICc:", x$results$AICc, " BIC:", x$results$BIC, "\n")
-    #cat("AIC:", x$results$AIC, "\n")
-    #cat("AICc:", x$results$AICc, "\n")
-    #cat("BIC:", x$results$BIC, "\n")
     #cat("Residual sum of squares:", x$results$rss, "\n")
     #cat("Quasi-global R2:", (1 - (x$results$rss/x$gTSS)), "\n")
   }
